@@ -1,4 +1,4 @@
-/* $PostgreSQL: pgsql/src/interfaces/ecpg/ecpglib/execute.c,v 1.38.4.11 2007/02/06 09:42:08 meskes Exp $ */
+/* $PostgreSQL: pgsql/src/interfaces/ecpg/ecpglib/execute.c,v 1.43.2.10 2007/02/27 13:27:05 meskes Exp $ */
 
 /*
  * The aim is to get a simpler inteface to the database routines.
@@ -38,39 +38,50 @@
 static char *
 quote_postgres(char *arg, int lineno)
 {
-	char	   *res = (char *) ECPGalloc(2 * strlen(arg) + 3, lineno);
-	int			i,
-				ri = 0;
+	char	*res;
+	int	error;
+	size_t  length;
+	size_t  escaped_len;
+	size_t  buffer_len;
 
+	/*
+	 * if quote is false we just need to store things in a descriptor they
+	 * will be quoted once they are inserted in a statement
+	 */
+	length = strlen(arg);
+	buffer_len = 2 * length + 1;
+	res = (char *) ECPGalloc(buffer_len + 3, lineno);
 	if (!res)
 		return (res);
 
-	res[ri++] = '\'';
-
-	for (i = 0; arg[i]; i++, ri++)
+	error = 0;
+	escaped_len = PQescapeString(res+1, arg, buffer_len);
+	if (error)
 	{
-		switch (arg[i])
-		{
-			case '\'':
-				res[ri++] = '\'';
-				break;
-			case '\\':
-				res[ri++] = '\\';
-				break;
-			default:
-				;
-		}
-		res[ri] = arg[i];
+		ECPGfree(res);
+		return NULL;
+	}
+	if (length == escaped_len)
+	{
+		res[0] = res[escaped_len+1] = '\'';
+		res[escaped_len+2] = '\0';
+	}
+	else
+	{
+		/* 
+		 * We don't know if the target database is using
+		 * standard_conforming_strings, so we always use E'' strings.
+		 */
+		memmove(res+2, res+1, escaped_len);
+		res[0] = ESCAPE_STRING_SYNTAX;
+		res[1] = res[escaped_len+2] = '\'';
+		res[escaped_len+3] = '\0';
 	}
-
-	res[ri++] = '\'';
-	res[ri] = '\0';
-
 	ECPGfree(arg);
 	return res;
 }
 
-#if defined(__GNUC__) && (defined (__powerpc__) || defined(__AMD64__) || defined(__x86_64__))
+#if defined(__GNUC__) && (defined (__powerpc__) || defined(__amd64__) || defined(__x86_64__))
 #define APREF ap
 #else
 #define APREF *ap
@@ -117,8 +128,7 @@ ECPGget_variable(va_list APREF, enum ECPGttype type, struct variable * var, bool
 			var->ind_value = var->ind_pointer;
 
 		/*
-		 * negative values are used to indicate an array without given
-		 * bounds
+		 * negative values are used to indicate an array without given bounds
 		 */
 		/* reset to zero for us */
 		if (var->ind_arrsize < 0)
@@ -274,9 +284,8 @@ ECPGis_type_an_array(int type, const struct statement * stmt, const struct varia
 	if ((stmt->connection->cache_head) == NULL)
 	{
 		/*
-		 * Text like types are not an array for ecpg, but postgres counts
-		 * them as an array. This define reminds you to not 'correct'
-		 * these values.
+		 * Text like types are not an array for ecpg, but postgres counts them
+		 * as an array. This define reminds you to not 'correct' these values.
 		 */
 #define not_an_array_in_ecpg ECPG_ARRAY_NONE
 
@@ -482,7 +491,7 @@ ECPGstore_result(const PGresult *results, int act_field,
 			int			len = strlen(PQgetvalue(results, act_tuple, act_field)) + 1;
 
 			if (!ECPGget_data(results, act_tuple, act_field, stmt->lineno,
-						 var->type, var->ind_type, current_data_location,
+							  var->type, var->ind_type, current_data_location,
 							  var->ind_value, len, 0, var->ind_offset, isarray, stmt->compat, stmt->force_indicator))
 				status = false;
 			else
@@ -517,8 +526,8 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 	char	   *newcopy = NULL;
 
 	/*
-	 * arrays are not possible unless the attribute is an array too FIXME:
-	 * we do not know if the attribute is an array here
+	 * arrays are not possible unless the attribute is an array too FIXME: we
+	 * do not know if the attribute is an array here
 	 */
 #if 0
 	if (var->arrsize > 1 &&...)
@@ -792,8 +801,7 @@ ECPGstore_input(const int lineno, const bool force_indicator, const struct varia
 							sprintf(mallocedval + strlen(mallocedval), "%c,", (((char *) var->value)[element]) ? 't' : 'f');
 
 					/*
-					 * this is necessary since sizeof(C++'s
-					 * bool)==sizeof(int)
+					 * this is necessary since sizeof(C++'s bool)==sizeof(int)
 					 */
 					else if (var->offset == sizeof(int))
 						for (element = 0; element < var->arrsize; element++)
@@ -1140,10 +1148,9 @@ ECPGexecute(struct statement * stmt)
 	copiedquery = ECPGstrdup(stmt->command, stmt->lineno);
 
 	/*
-	 * Now, if the type is one of the fill in types then we take the
-	 * argument and enter that in the string at the first %s position.
-	 * Then if there are any more fill in types we fill in at the next and
-	 * so on.
+	 * Now, if the type is one of the fill in types then we take the argument
+	 * and enter that in the string at the first %s position. Then if there
+	 * are any more fill in types we fill in at the next and so on.
 	 */
 	var = stmt->inlist;
 
@@ -1158,14 +1165,14 @@ ECPGexecute(struct statement * stmt)
 		tobeinserted = NULL;
 
 		/*
-		 * A descriptor is a special case since it contains many variables
-		 * but is listed only once.
+		 * A descriptor is a special case since it contains many variables but
+		 * is listed only once.
 		 */
 		if (var->type == ECPGt_descriptor)
 		{
 			/*
-			 * We create an additional variable list here, so the same
-			 * logic applies.
+			 * We create an additional variable list here, so the same logic
+			 * applies.
 			 */
 			struct variable desc_inlist;
 			struct descriptor *desc;
@@ -1236,8 +1243,8 @@ ECPGexecute(struct statement * stmt)
 		if (tobeinserted)
 		{
 			/*
-			 * Now tobeinserted points to an area that is to be inserted
-			 * at the first %s
+			 * Now tobeinserted points to an area that is to be inserted at
+			 * the first %s
 			 */
 			if (!(newcopy = (char *) ECPGalloc(strlen(copiedquery)
 											 + strlen(tobeinserted)
@@ -1251,8 +1258,8 @@ ECPGexecute(struct statement * stmt)
 			if ((p = next_insert(newcopy + hostvarl)) == NULL)
 			{
 				/*
-				 * We have an argument but we dont have the matched up
-				 * string in the string
+				 * We have an argument but we dont have the matched up string
+				 * in the string
 				 */
 				ECPGraise(stmt->lineno, ECPG_TOO_MANY_ARGUMENTS,
 						  ECPG_SQLSTATE_USING_CLAUSE_DOES_NOT_MATCH_PARAMETERS,
@@ -1267,8 +1274,8 @@ ECPGexecute(struct statement * stmt)
 				hostvarl = strlen(newcopy);
 
 				/*
-				 * The strange thing in the second argument is the rest of
-				 * the string from the old string
+				 * The strange thing in the second argument is the rest of the
+				 * string from the old string
 				 */
 				strcat(newcopy,
 					   copiedquery
@@ -1277,9 +1284,9 @@ ECPGexecute(struct statement * stmt)
 			}
 
 			/*
-			 * Now everything is safely copied to the newcopy. Lets free
-			 * the oldcopy and let the copiedquery get the var->value from
-			 * the newcopy.
+			 * Now everything is safely copied to the newcopy. Lets free the
+			 * oldcopy and let the copiedquery get the var->value from the
+			 * newcopy.
 			 */
 			if (malloced)
 			{
